/*
 * LaunchPad 1.11 public code release - October 1, 1997
 *  Official LaunchPad page: http://www.nwlink.com/~emilyk/LaunchPad
 *
 *  This code compiles with the EXCELLENT GCC 0.4.0 Win32 port for PalmPilot.
 *  Find info about it at http://www.shoppersmart.com/jlehett/gccwin32.html
 *  Does not compile under the 0.5.0 port because of PalmOS 2.0 differences.
 *
 * Disclaimers and other legal stuff:
 *  This code is provided AS-IS; there is NO WARRANTY, express or implied.
 *  The user assumes full responsibility and liability for any use of
 *  this code.
 *
 *  Copyright (C)1997 Eric Kenslow.
 *  All Rights Reserved by Eric Kenslow, except as noted below.
 *
 *  Permission to distribute this package complete and unmodified
 *  for personal use via the internet for no charge (other than reasonable
 *  connection charges) is granted to the public.
 *  Distribution for commercial purposes via any other medium, including but
 *  not limited to CD-ROM, floppy, or printed material, in whole or in part,
 *  requires express permission from Eric Kenslow (erick@metainfo.com).
 *
 *  Modified versions may be distributed with this notice intact, provided
 *  that said versions are clearly marked as modified code. The above
 *  provisions also apply to any such modified packages.
 *
 *  Permission is also granted to the public to use PARTS of this code in
 *  other projects, with no special notice required unless the source code
 *  is part of the distribution, in which case this notice must remain
 *  conspicuous in the code that was used.
 *
 * Note: These terms are intended to protect me, not make this package less
 *       useful. If you have a special need or use for this package, contact
 *       me and we'll work something out.
 *                                                - Eric
 *                                                  erick@metainfo.com
 *
 */


/*****************************************************************************
*
*   Module name:  AppMan
*   Project:      LaunchPad
*
*   Created:      3/30/1997  Eric Kenslow
*
*   Description:  Code & data for the apps window manager
*
*****************************************************************************/

#include "LaunchPad.h"
#include <featuremgr.h>

// Modular types

typedef struct tagAppInfo
{
  ULong         idApp;
  Byte          iTab;

  char          szTitle   [ 32 ];

  Boolean       fHasBitmap;
  ULong         rgbBitmap [ 144 / sizeof( ULong ) ];  // Hello magic number!

  LocalID       idDB;
  UInt          iCardDB;
  UInt          iVer;

  RectangleType rc;
} LauncherInfoType, *LauncherInfoPtr;

typedef struct tagAppItem
{
  struct tagAppItem*  pNext;

  UInt    iRecord;

  LauncherInfoPtr pInfo;
} LauncherItemType, *LauncherItemPtr;

typedef struct tagAppCache
{
  ULong   idApp;
  UInt    iCardDB;
  LocalID idDB;
  UInt    iVer;
} AppEnumType, *AppEnumPtr;

// End modular types

// Modular constants
#define mg_dyFirstApp ((SWord)1)
// End modular constants

// Modular globals

static  WinHandle       mg_hwinApps;
static  RectangleType   mg_rcApps, mg_rcAppsScreen;
static  RectangleType   mg_rcDrag;
static  LauncherItemPtr mg_pAppHead;
static  LauncherItemPtr mg_pLastHit;

// End modular globals

/*** LoadAppsDB **************************************************************
*
*   Purpose:   Loads all apps we already have in our DB
*
*   Returns:   true on success
*
*   Created:   4/10/1997  Eric Kenslow
*****************************************************************************/

static Boolean LoadAppsDB( UInt cApps, AppEnumPtr prgApps,
                            BooleanPtr pfRecalcRects )
{
  Byte    i;
  Boolean fSuccess = true;

  mg_pAppHead = 0;
  for( i = 0; fSuccess && i < DBRecordCount( apps ); i++ )
    {
    LauncherItemPtr pCur = MemPtrNew( sizeof( LauncherItemType ) );
    if( pCur )
      {
      pCur->pNext = mg_pAppHead;
      mg_pAppHead = pCur;

      pCur->pInfo = DBGetRecord( apps, i );
      if( !(pCur->pInfo) )
        fSuccess = false;
      else
        {
        Boolean fExists = false;
        UInt    iApp = 0;
        // Verify that the app still exists
        while( !fExists && iApp < cApps )
          {
          if( prgApps[ iApp ].idApp == pCur->pInfo->idApp )
            fExists = true;
          else
            iApp++;
          }
        if( !fExists || mg_pAppHead->pInfo->idApp == g_dwCreatorID )
          {
          // App doesn't exist; remove it from our list and DB
          LauncherItemPtr pT = mg_pAppHead;
          mg_pAppHead = mg_pAppHead->pNext;
          DBReleaseRecord( apps, i, pT->pInfo );
          DBDeleteRecord( apps, i );
          i--;
          MemPtrFree( pT );
          *pfRecalcRects = true;
          }
        else
          {
          // Pop the app's info into our list
          pCur->iRecord = i;
          }
        }
      }
    else
      fSuccess = false;
    }

  return fSuccess;
}

/*** InDB ********************************************************************
*
*   Purpose:   Checks if a given app is in our app list already
*
*   Returns:   true if it is
*
*   Created:   4/10/1997  Eric Kenslow
*****************************************************************************/

static Boolean InDB( ULong idApp, LauncherItemPtr* ppApp )
{
  LauncherItemPtr pCur = mg_pAppHead;
  while( pCur )
    {
    if( pCur->pInfo->idApp == idApp )
      {
      *ppApp = pCur;
      return true;
      }
    pCur = pCur->pNext;
    }
  return false;
}

/*** AppManRecalcRects *******************************************************
*
*   Purpose:   Recalculates app rectangles
*
*   Returns:   Nothing
*
*   Created:   4/10/1997  Eric Kenslow
*****************************************************************************/

void AppManRecalcRects( void )
{
  LauncherItemPtr pCurHead = mg_pAppHead;
  while( pCurHead )
    {
    LauncherItemPtr pCur = pCurHead;
    RectangleType rcCur = { { 0, mg_dyFirstApp },
                            { 160 / g_prefs.data.cAppColumns, 33 } };
    ErrFatalDisplayIf( g_prefs.data.cAppColumns < 2, "OH NO!" );
    ErrFatalDisplayIf( rcCur.extent.x < 10, "oh no!" );

    while( pCur )
      {
      if( pCur == pCurHead ||
          pCur->pInfo->iTab == pCurHead->pInfo->iTab )
        {
        // REVIEW: Pointer arithmetic- no offsetof() macro
        DmWrite( pCurHead->pInfo,
                  (ULong)&(pCurHead->pInfo->rc) - (ULong)pCurHead->pInfo,
                  &rcCur, sizeof( rcCur ) );

        RctOffsetRectangle( &rcCur, rcCur.extent.x, 0 );
        if( (rcCur.topLeft.x + rcCur.extent.x) > 160 )
          {
          RctOffsetRectangle( &rcCur, 0, rcCur.extent.y );
          rcCur.topLeft.x = 0;
          }
        }
      pCur = pCur->pNext;
      }

    pCurHead = pCurHead->pNext;
    }
}

/*** SortAppsList ************************************************************
*
*   Purpose:   Sorts the list of apps by name, using an insertion sort
*
*   Returns:   Nothing
*
*   Created:   4/10/1997  Eric Kenslow
*****************************************************************************/

static void SortAppsList( Boolean fRecalcRects )
{
  LauncherItemPtr pNewHead = 0;

  while( mg_pAppHead )
    {
    LauncherItemPtr pCur = mg_pAppHead;

    // Remove the current item from the list
    mg_pAppHead = mg_pAppHead->pNext;

    if( !pNewHead )
      {
      pCur->pNext = 0;
      pNewHead    = pCur;
      }
    else
      {
      // Find the insertion point
      LauncherItemPtr pInsert = pNewHead, pPrev = 0;
      while( pInsert &&
             StrCompare( pInsert->pInfo->szTitle,
                         pCur->pInfo->szTitle ) > 0 )
        {
        pPrev   = pInsert;
        pInsert = pInsert->pNext;
        }

      if( pInsert == 0 )        // End
        {
        pPrev->pNext = pCur;
        pCur->pNext  = 0;
        }
      else if( pPrev == 0 )     // Head
        {
        pCur->pNext = pNewHead;
        pNewHead    = pCur;
        }
      else                      // Middle
        {
        pCur->pNext  = pInsert;
        pPrev->pNext = pCur;
        }
      }
    }

  mg_pAppHead = pNewHead;

  if( fRecalcRects )
    AppManRecalcRects();
}

/*** GetAppResources *********************************************************
*
*   Purpose:   Loads an app's resources
*
*   Returns:   Nothing
*
*   Created:   4/20/1997  Eric Kenslow
*****************************************************************************/

static Boolean GetAppResources( UInt iCardDB, LocalID idDB,
                                 LauncherInfoPtr pInfo, Boolean fDBPtr )
{
  Boolean fSuccess = true;

  // See if the app has a tAIN (App name) resource
  DmOpenRef hDB = DmOpenDatabase( iCardDB, idDB, dmModeReadOnly );
  if( hDB )
    {
    VoidHand  hRes = DmGet1Resource( (ULong)'tAIN', 1000 );
    if( hRes )
      {
      CharPtr pAIN = MemHandleLock( hRes );
      if( pAIN )
        {
        if( fDBPtr )
          // REVIEW: pointer arithmetic
          DmStrCopy( pInfo, (ULong)pInfo->szTitle - (ULong)pInfo, pAIN );
        else
          StrCopy( pInfo->szTitle, pAIN );
        MemHandleUnlock( hRes );
        }
      DmReleaseResource( hRes );
      }

    // Now get the tAIB (App icon) resource
    hRes = DmGet1Resource( (ULong)'tAIB', 1000 );
    if( hRes )
      {
      BytePtr pAIB = MemHandleLock( hRes );
      if( pAIB )
        {
        if( fDBPtr )
          {
          Boolean fDummy = true;
          // REVIEW: Pointer arithmetic
          DmWrite( pInfo, (ULong)&(pInfo->fHasBitmap) - (ULong)pInfo,
                    &fDummy, sizeof( fDummy ) );
          DmWrite( pInfo, (ULong)pInfo->rgbBitmap - (ULong)pInfo,
                    pAIB, sizeof( pInfo->rgbBitmap ) );
          }
        else
          {
          pInfo->fHasBitmap = true;
          MemMove( pInfo->rgbBitmap, pAIB, sizeof( pInfo->rgbBitmap ) );
          }
        MemHandleUnlock( hRes );
        }
      DmReleaseResource( hRes );
      }
    else
      {
      if( fDBPtr )
        {
        Boolean fDummy = false;
        // REVIEW: Pointer arithmetic
        DmWrite( pInfo, (ULong)&(pInfo->fHasBitmap) - (ULong)pInfo,
                  &fDummy, sizeof( fDummy ) );
//        DmSet( pInfo, (ULong)&(pInfo->fHasBitmap) - (ULong)pInfo,
//                sizeof( pInfo->rgbBitmap ), 0 );
        }
      else
        {
        pInfo->fHasBitmap = false;
//        MemSet( pInfo->rgbBitmap, sizeof( pInfo->rgbBitmap ), 0 );
        }
      }
    DmCloseDatabase( hDB );
    }
  else
    fSuccess = false;
  return fSuccess;
}

/*** InitAppsList ************************************************************
*
*   Purpose:   Gets any apps that weren't in our DB
*
*   Returns:   true if success
*
*   Created:   4/10/1997  Eric Kenslow
*****************************************************************************/

static Boolean InitAppsList( UInt cApps, AppEnumPtr prgApps,
                              BooleanPtr pfRecalcRects,
                              Boolean fForceRefresh )
{
  Boolean fSuccess = true;
  UInt    iApp;

  for( iApp = 0; fSuccess && iApp < cApps; iApp++ )
    {
    UInt    iCardDB = prgApps[ iApp ].iCardDB;
    LocalID idDB    = prgApps[ iApp ].idDB;
    LauncherItemPtr pApp;

    if( prgApps[ iApp ].idApp == g_dwCreatorID )
      continue;

    if( !InDB( prgApps[ iApp ].idApp, &pApp ) )
      {
      LauncherInfoType  infoT;
      UInt  iRecord;

      DmDatabaseInfo( iCardDB, idDB,
                       infoT.szTitle, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 );

      fSuccess = GetAppResources( iCardDB, idDB, &infoT, false );

      infoT.iTab    = 0;
      infoT.idApp   = prgApps[ iApp ].idApp;
      infoT.idDB    = prgApps[ iApp ].idDB;
      infoT.iCardDB = prgApps[ iApp ].iCardDB;
      iRecord = dbAddAtEnd;
      if( DBAddRecord( apps, &iRecord, &infoT, sizeof( infoT ) ) )
        {
        LauncherItemPtr pItem = MemPtrNew( sizeof( LauncherItemType ) );

        if( pItem )
          {
          pItem->pNext   = mg_pAppHead;
          pItem->iRecord = iRecord;
          mg_pAppHead = pItem;

          pItem->pInfo = DBGetRecord( apps, iRecord );
          }
        else
          fSuccess = false;
        }
      else
        fSuccess = false;

      *pfRecalcRects = true;
      }
    // Update new info, if necessary
    // Checks are in order of likelihood
    else if( fForceRefresh ||
             prgApps[ iApp ].idDB    != pApp->pInfo->idDB ||
             prgApps[ iApp ].iVer    != pApp->pInfo->iVer ||
             prgApps[ iApp ].iCardDB != pApp->pInfo->iCardDB )
      {
      // App moved, or newer version! Update the info
      DmWrite( pApp->pInfo, (ULong)&(pApp->pInfo->idDB) - (ULong)pApp->pInfo,
                &idDB, sizeof( idDB ) );
      DmWrite( pApp->pInfo,
                (ULong)&(pApp->pInfo->iCardDB) - (ULong)pApp->pInfo,
                &iCardDB, sizeof( iCardDB ) );
      DmWrite( pApp->pInfo,
                (ULong)&(pApp->pInfo->iVer) - (ULong)pApp->pInfo,
                &(prgApps[ iApp ].iVer), sizeof( prgApps[ iApp ].iVer ) );
      GetAppResources( iCardDB, idDB, pApp->pInfo, true );
      }
    }

  return fSuccess;
}

/*** EnumAllApps *************************************************************
*
*   Purpose:   Enumerates all apps on this pilot into a buffer
*
*   Returns:   Nothing
*
*   Created:   4/13/1997  Eric Kenslow
*****************************************************************************/

static void EnumAllApps( UIntPtr pcApps, VoidHand* phrgApps )
{
  AppEnumPtr        prgApps;
  UInt              iCard;

  *pcApps = 0;
  *phrgApps = MemHandleNew( sizeof( AppEnumType ) );
  prgApps = MemHandleLock( *phrgApps );
  for( iCard = 0; iCard < 2; iCard++ )
    {
    UInt  cDBs = DmNumDatabases( iCard );
    UInt  iDB;

    MemHandleUnlock( *phrgApps );
    MemHandleResize( *phrgApps, sizeof( AppEnumType ) * (*pcApps + cDBs) );
    prgApps = MemHandleLock( *phrgApps );

    for( iDB = 0; iDB < cDBs; iDB++ )
      {
      ULong   dbType;
      LocalID idDB = DmGetDatabase( iCard, iDB );
      DmDatabaseInfo( iCard, idDB, 0, 0, &(prgApps[ *pcApps ].iVer),
                       0, 0, 0, 0, 0, 0,
                       &dbType, &(prgApps[ *pcApps ].idApp) );
      if( dbType == (ULong)'appl' )
        {
        prgApps[ *pcApps ].iCardDB = iCard;
        prgApps[ *pcApps ].idDB    = idDB;
        (*pcApps)++;
        }
      }
    }
}

/*** AppManInit **************************************************************
*
*   Purpose:   Initializes the apps window manager
*
*   Returns:   true if successful
*
*   Created:   3/30/1997  Eric Kenslow
*****************************************************************************/

Boolean AppManInit( RectanglePtr prc )
{
  VoidHand    hrgApps = 0;
  AppEnumPtr  prgApps = 0;
  UInt        cApps = 0;
  Boolean     fSuccess = true, fRecalcRects = false;
  Boolean     fForceRefresh = false;
  Word        err;
  DWord       dwT;

  if( FtrGet( g_dwCreatorID, g_iFtrRefresh, &dwT ) == 0 )
    {
    fForceRefresh = true;
    FtrUnregister( g_dwCreatorID, g_iFtrRefresh );
    }

  RctSetRectangle( &mg_rcDrag, 0, 0, 0, 0 );

  EnumAllApps( &cApps, &hrgApps );
  prgApps = MemHandleLock( hrgApps );
  ErrFatalDisplayIf( !prgApps, "Oh no!" );
  if( LoadAppsDB( cApps, prgApps, &fRecalcRects ) &&
      InitAppsList( cApps, prgApps, &fRecalcRects, fForceRefresh ) )
    SortAppsList( fRecalcRects );
  else
    fSuccess = false;
  if( hrgApps )
    {
    MemHandleUnlock( hrgApps );
    MemHandleFree( hrgApps );
    }

  mg_pLastHit = 0;

  if( prc )
    {
    mg_hwinApps = WinCreateWindow( prc, rectangleFrame, false, false, &err );
    mg_rcApps       = *prc;
    mg_rcAppsScreen = *prc;
    }
  else
    {
    mg_rcApps = mg_rcAppsScreen;
    mg_hwinApps = WinCreateWindow( &mg_rcApps, rectangleFrame, false, false,
                                    &err );
    }

  if( err )
    fSuccess = false;
  else
    {
    WinHandle hwinDraw = WinGetDrawWindow();
    WinSetDrawWindow( mg_hwinApps );
    WinDisplayToWindowPt( &(mg_rcApps.topLeft.x), &(mg_rcApps.topLeft.y) );
    WinSetDrawWindow( hwinDraw );
    }

  return fSuccess;
}

/*** AppManCleanup ***********************************************************
*
*   Purpose:   Cleans up after the apps window manager
*
*   Returns:   Nothing
*
*   Created:   3/30/1997  Eric Kenslow
*****************************************************************************/

void AppManCleanup( void )
{
  LauncherItemPtr pCur = mg_pAppHead;
  mg_pAppHead = 0;
  while( pCur )
    {
    LauncherItemPtr pT = pCur;
    pCur = pCur->pNext;
    DBReleaseRecord( apps, pT->iRecord, pT->pInfo );
    MemPtrFree( pT );
    }

  WinDeleteWindow( mg_hwinApps, false );
}

/*** AppManDrawApps **********************************************************
*
*   Purpose:   Draws the apps window
*
*   Returns:   Nothing
*
*   Created:   4/9/1997  Eric Kenslow
*****************************************************************************/

void AppManDrawApps( void )
{
  LauncherItemPtr pCur;
  Word            cyText;
  WinHandle hwinDraw = WinGetDrawWindow();
  WinSetDrawWindow( mg_hwinApps );

  FntSetFont( 0 );  // Normal
  cyText = FntCharHeight();

  WinEraseRectangle( &mg_rcApps, 0 );

  pCur = mg_pAppHead;
  while( pCur )
    {
    if( pCur->pInfo->iTab == g_prefs.iTabCur )
      {
      Word  cbTitle = StrLen( pCur->pInfo->szTitle );
      SWord  dxText = FntCharsWidth( pCur->pInfo->szTitle,
                                      cbTitle ) / 2;
      UInt      dxBmp = (pCur->pInfo->rc.extent.x / 2) - (32 / 2);

      if( pCur->pInfo->fHasBitmap )
        WinDrawBitmap( (BitmapPtr)pCur->pInfo->rgbBitmap,
                          pCur->pInfo->rc.topLeft.x + dxBmp,
                          pCur->pInfo->rc.topLeft.y );

      WinDrawChars( pCur->pInfo->szTitle, cbTitle,
                     (pCur->pInfo->rc.topLeft.x +
                      (pCur->pInfo->rc.extent.x / 2)) - dxText,
                     (pCur->pInfo->rc.topLeft.y +
                      pCur->pInfo->rc.extent.y) - cyText );
      }

    pCur = pCur->pNext;
    }

  {
  RectangleType rcT;
  RctCopyRectangle( &mg_rcApps, &rcT );
  rcT.extent.x  -= 2;
  rcT.extent.y  -= 1;
  rcT.topLeft.x += 1;
  WinDrawRectangleFrame( roundFrame, &rcT );
  }
  WinSetDrawWindow( hwinDraw );
}

/*** AppManHitTest ***********************************************************
*
*   Purpose:   Hit-tests a specified point
*
*   Returns:   true if an app is at that point
*
*   Created:   4/11/1997  Eric Kenslow
*****************************************************************************/

Boolean AppManHitTest( SWord x, SWord y, Boolean fDownStroke )
{
  LauncherItemPtr pCur = mg_pAppHead;

  WinHandle hwinDraw = WinGetDrawWindow();
  WinSetDrawWindow( mg_hwinApps );

  WinDisplayToWindowPt( &x, &y );

  if( !fDownStroke && mg_pLastHit )
    {
    // Clear the old hilite rect
    WinInvertRectangle( &(mg_pLastHit->pInfo->rc), 5 );
    }
  else
    mg_pLastHit = 0;

  WinSetDrawWindow( hwinDraw );

  while( pCur )
    {
    if( pCur->pInfo->iTab == g_prefs.iTabCur &&
        RctPtInRectangle( x, y, &(pCur->pInfo->rc) ) )
      {
      if( !RctPtInRectangle( x, y, &mg_rcApps ) )
        return false;

      if( fDownStroke )
        {
        // Draw the new hilite rect
        WinHandle hwinDraw = WinGetDrawWindow();
        mg_pLastHit = pCur;
        WinSetDrawWindow( mg_hwinApps );
        WinInvertRectangle( &(mg_pLastHit->pInfo->rc), 5 );
        WinSetDrawWindow( hwinDraw );
        }
      else
        if( pCur != mg_pLastHit )
          mg_pLastHit = 0;

      return true;
      }

    pCur = pCur->pNext;
    }

  return false;
}

/*** AppManLaunchLast ********************************************************
*
*   Purpose:   Launches the last-hit app
*
*   Returns:   true if successful
*
*   Created:   4/11/1997  Eric Kenslow
*****************************************************************************/

Boolean AppManLaunchLast( void )
{
  if( !mg_pLastHit )
    return false;
  return SysUIAppSwitch( mg_pLastHit->pInfo->iCardDB,
                         mg_pLastHit->pInfo->idDB,
                         sysAppLaunchCmdNormalLaunch, 0 )
         == 0 ? true : false;
}

/*** AppManScroll ************************************************************
*
*   Purpose:   Scrolls the apps up or down
*
*   Returns:   true if successful
*
*   Created:   4/11/1997  Eric Kenslow
*****************************************************************************/

Boolean AppManScroll( DirectionType dir )
{
  Boolean fSuccess = AppManCanScroll( dir );

  if( fSuccess )
    {
    LauncherItemPtr pCur = mg_pAppHead;
    Word    dy = mg_pAppHead->pInfo->rc.extent.y;
    if( g_prefs.flags.fPageScroll )
      {
      dy *= (mg_rcApps.extent.y / dy);
      }
    if( dir == down )
      dy = -dy;

    while( pCur )
      {
      if( pCur->pInfo->iTab == g_prefs.iTabCur )
        {
        SWord yNew = pCur->pInfo->rc.topLeft.y + dy;
        // REVIEW: Pointer arithmetic
        DmWrite( pCur->pInfo,
                  (ULong)&(pCur->pInfo->rc.topLeft.y) - (ULong)pCur->pInfo,
                  &yNew, sizeof( yNew ) );
        }
      pCur = pCur->pNext;
      }
    }

  return fSuccess;
}

/*** AppManCanScroll *********************************************************
*
*   Purpose:   Queries if we can scroll in a given direction
*
*   Returns:   true if we can
*
*   Created:   4/12/1997  Eric Kenslow
*****************************************************************************/

Boolean AppManCanScroll( DirectionType dir )
{
  LauncherItemPtr pCur = mg_pAppHead;
  Boolean fSuccess = false;
  Word    dy = mg_pAppHead->pInfo->rc.extent.y;

  if( dir == down )
    dy = -dy;

  while( !fSuccess && pCur )
    {
    if( pCur->pInfo->iTab == g_prefs.iTabCur )
      {
      if( dir == up &&
          pCur->pInfo->rc.topLeft.y + dy == mg_dyFirstApp )
        // OK, at least 1 will be on the top line
        fSuccess = true;
      else if( dir == down &&
               pCur->pInfo->rc.topLeft.y +
                pCur->pInfo->rc.extent.y > mg_rcApps.extent.y )
        // OK, at least 1 will be scrolled into view
        fSuccess = true;
      }
    pCur = pCur->pNext;
    }

  return fSuccess;
}

/*** AppManMoveLast **********************************************************
*
*   Purpose:   Moves the last-hit app to a new tab
*
*   Returns:   true if an app was moved
*
*   Created:   4/12/1997  Eric Kenslow
*****************************************************************************/

Boolean AppManMoveLast( Byte iTabNew )
{
  if( mg_pLastHit )
    {
    // REVIEW: Pointer arithmetic
    DmWrite( mg_pLastHit->pInfo,
              (ULong)&(mg_pLastHit->pInfo->iTab) - (ULong)mg_pLastHit->pInfo,
              &iTabNew, sizeof( iTabNew ) );
    AppManRecalcRects();
    mg_pLastHit = 0;

    return true;
    }
  return false;
}

/*** AppManClearTab **********************************************************
*
*   Purpose:   Removes all items from the current tab, preparing for tab
*               deletion
*
*   Returns:   Nothing
*
*   Created:   4/12/1997  Eric Kenslow
*****************************************************************************/

void AppManClearTab( void )
{
  LauncherItemPtr pCur = mg_pAppHead;
  while( pCur )
    {
    Byte  iTabNew = pCur->pInfo->iTab;
    if( iTabNew == g_prefs.iTabCur )
      iTabNew = 0;
    else if( iTabNew > g_prefs.iTabCur )
      iTabNew--;
    // REVIEW: Pointer arithmetic
    DmWrite( pCur->pInfo, (ULong)&(pCur->pInfo->iTab) - (ULong)pCur->pInfo,
              &iTabNew, sizeof( iTabNew ) );

    pCur = pCur->pNext;
    }

  AppManRecalcRects();
}

/*** AppManDrag **************************************************************
*
*   Purpose:   Draws the dragged app's icon
*
*   Returns:   Nothing
*
*   Created:   4/20/1997  Eric Kenslow
*****************************************************************************/

void AppManDrag( SWord x, SWord y )
{
  if( mg_pLastHit )
    {
    // Erase the old
    WinInvertRectangle( &mg_rcDrag, 5 );

    if( x >= 0 && y >= 0 )
      {
      // Draw the drag
      RectangleType rcT;
      RctCopyRectangle( &(mg_pLastHit->pInfo->rc), &rcT );
      RctCopyRectangle( &(mg_pLastHit->pInfo->rc), &mg_rcDrag );
      mg_rcDrag.topLeft.x = x - (mg_rcDrag.extent.x / 2);
      mg_rcDrag.topLeft.y = y - (mg_rcDrag.extent.y / 2);

      RctOffsetRectangle( &rcT, mg_rcAppsScreen.topLeft.x,
                                mg_rcAppsScreen.topLeft.y );
      if( !RctPtInRectangle( x, y, &rcT ) )
        WinInvertRectangle( &mg_rcDrag, 5 );
      else
        RctSetRectangle( &mg_rcDrag, 0, 0, 0, 0 );
      }
    else
      RctSetRectangle( &mg_rcDrag, 0, 0, 0, 0 );
    }
}

/*** AppManMoveTab ***********************************************************
*
*   Purpose:   Makes mods necessary after moving a tab
*
*   Returns:   Nothing
*
*   Created:   4/21/1997  Eric Kenslow
*****************************************************************************/

void AppManMoveTab( Byte iFrom, Byte iTo )
{
  LauncherItemPtr pCur = mg_pAppHead;
  while( pCur )
    {
    Byte    iT;
    BytePtr piSrc = 0;

    if( pCur->pInfo->iTab == iFrom )
      piSrc = &iTo;
    else if( pCur->pInfo->iTab >  iFrom &&
             pCur->pInfo->iTab <= iTo )
      {
      iT = pCur->pInfo->iTab - 1;
      piSrc = &iT;
      }
    else if( pCur->pInfo->iTab <  iFrom &&
             pCur->pInfo->iTab >= iTo )
      {
      iT = pCur->pInfo->iTab + 1;
      piSrc = &iT;
      }

    if( piSrc )
      // REVIEW: pointer arithmetic
      DmWrite( pCur->pInfo, (ULong)&(pCur->pInfo->iTab) - (ULong)pCur->pInfo,
                piSrc, sizeof( *piSrc ) );

    pCur = pCur->pNext;
    }
  AppManRecalcRects();
}

