/*
 * LaunchPad 1.11 public code release - October 1, 1997
 *  Official LaunchPad page: http://www.nwlink.com/~emilyk/LaunchPad
 *
 *  This code compiles with the EXCELLENT GCC 0.4.0 Win32 port for PalmPilot.
 *  Find info about it at http://www.shoppersmart.com/jlehett/gccwin32.html
 *  Does not compile under the 0.5.0 port because of PalmOS 2.0 differences.
 *
 * Disclaimers and other legal stuff:
 *  This code is provided AS-IS; there is NO WARRANTY, express or implied.
 *  The user assumes full responsibility and liability for any use of
 *  this code.
 *
 *  Copyright (C)1997 Eric Kenslow.
 *  All Rights Reserved by Eric Kenslow, except as noted below.
 *
 *  Permission to distribute this package complete and unmodified
 *  for personal use via the internet for no charge (other than reasonable
 *  connection charges) is granted to the public.
 *  Distribution for commercial purposes via any other medium, including but
 *  not limited to CD-ROM, floppy, or printed material, in whole or in part,
 *  requires express permission from Eric Kenslow (erick@metainfo.com).
 *
 *  Modified versions may be distributed with this notice intact, provided
 *  that said versions are clearly marked as modified code. The above
 *  provisions also apply to any such modified packages.
 *
 *  Permission is also granted to the public to use PARTS of this code in
 *  other projects, with no special notice required unless the source code
 *  is part of the distribution, in which case this notice must remain
 *  conspicuous in the code that was used.
 *
 * Note: These terms are intended to protect me, not make this package less
 *       useful. If you have a special need or use for this package, contact
 *       me and we'll work something out.
 *                                                - Eric
 *                                                  erick@metainfo.com
 *
 */


/*****************************************************************************
*
*   Module name:  Database
*   Project:      LaunchPad
*
*   Created:      3/30/1997  Eric Kenslow
*
*   Description:  Code & data for the database interface
*
*****************************************************************************/

#include "LaunchPad.h"

// Modular constants/macros

#define dwTabsDBType  ((ULong)'Tabs')
#define dwAppsDBType  ((ULong)'DATA')

// End modular constants

// Modular functions

static  DmOpenRef OpenOrCreateDatabase  ( CharPtr, ULong );

// End modular functions

// Modular globals

static  DmOpenRef mg_hDB[ cDBs ];

// End modular globals

/*** OpenOrCreateDatabase ****************************************************
*
*   Purpose:   Opens a database, or creates it if it doesn't exist
*
*   Returns:   Dm Open Ref to the DB, or 0 if error
*
*   Created:   3/30/1997  Eric Kenslow
*****************************************************************************/

static DmOpenRef OpenOrCreateDatabase( CharPtr szDBName, ULong dbType )
{
  DmOpenRef hDB = DmOpenDatabaseByTypeCreator( dbType, g_dwCreatorID,
                                                dmModeReadWrite );
  if( !hDB )
    {
    if( DmCreateDatabase( 0, szDBName, g_dwCreatorID, dbType, false ) == 0 )
      hDB = DmOpenDatabaseByTypeCreator( dbType, g_dwCreatorID,
                                          dmModeReadWrite );
    }
  return hDB;
}

/*** DBInit ******************************************************************
*
*   Purpose:   Initializes the database system
*
*   Returns:   true if successful
*
*   Created:   3/30/1997  Eric Kenslow
*****************************************************************************/

Boolean DBInit( void )
{
  Boolean fSuccess = true;

  mg_hDB[ tabs ] = OpenOrCreateDatabase( "LaunchPad Tabs", dwTabsDBType );
  fSuccess = mg_hDB[ tabs ] ? true : false;

  mg_hDB[ apps ] = OpenOrCreateDatabase( "LaunchPad Data", dwAppsDBType );
  fSuccess = fSuccess && (mg_hDB[ apps ] ? true : false);

  return fSuccess;
}

/*** DBCleanup ***************************************************************
*
*   Purpose:   Cleans up after the database system
*
*   Returns:   Nothing
*
*   Created:   3/30/1997  Eric Kenslow
*****************************************************************************/

void DBCleanup( void )
{
  UInt  i;
  for( i = 0; i < cDBs; i++ )
    if( mg_hDB[ i ] )
      {
      DmCloseDatabase( mg_hDB[ i ] );
      mg_hDB[ i ] = 0;
      }
}

/*** DBGetRecord *************************************************************
*
*   Purpose:   Gets a pointer to a record's data
*
*   Returns:   Pointer to the data, or 0 if failure
*
*   Created:   3/30/1997  Eric Kenslow
*****************************************************************************/

VoidPtr DBGetRecord( DBIndexType iDB, UInt iRec )
{
  VoidHand  hRec = DmGetRecord( mg_hDB[ iDB ], iRec );
  VoidPtr   pRec = 0;
  if( hRec )
    pRec = MemHandleLock( hRec );
  return pRec;
}

/*** DBAddRecord *************************************************************
*
*   Purpose:   Adds a record and sets its data
*
*   Returns:   true if successful
*
*   Created:   3/30/1997  Eric Kenslow
*****************************************************************************/

Boolean DBAddRecord( DBIndexType iDB, UIntPtr piRec, VoidPtr pRec, ULong cb )
{
  Boolean fSuccess = false;
  VoidHand  hRec = DmNewRecord( mg_hDB[ iDB ], piRec, cb );

  if( hRec )
    {
    VoidPtr pDBRec = MemHandleLock( hRec );
    DmWrite( pDBRec, 0, pRec, cb );
    MemHandleUnlock( hRec );
    DmReleaseRecord( mg_hDB[ iDB ], *piRec, true );

    fSuccess = true;
    }

  return fSuccess;
}

/*** DBReleaseRecord *********************************************************
*
*   Purpose:   Releases a record
*
*   Returns:   Nothing
*
*   Created:   3/30/1997  Eric Kenslow
*****************************************************************************/

void DBReleaseRecord( DBIndexType iDB, UInt iRec, VoidPtr precToUnlock )
{
  if( precToUnlock )
    MemPtrUnlock( precToUnlock );
  DmReleaseRecord( mg_hDB[ iDB ], iRec, true );
}

/*** DBDeleteRecord **********************************************************
*
*   Purpose:   Permanently deletes a record
*
*   Returns:   Nothing
*
*   Created:   4/12/1997  Eric Kenslow
*****************************************************************************/

void DBDeleteRecord( DBIndexType iDB, UInt iRec )
{
  DmRemoveRecord( mg_hDB[ iDB ], iRec );
}

/*** DBRecordCount ***********************************************************
*
*   Purpose:   Gets the # of records in a database
*
*   Returns:   The number of records
*
*   Created:   4/8/1997  Eric Kenslow
*****************************************************************************/

UInt DBRecordCount( DBIndexType iDB )
{
  return DmNumRecords( mg_hDB[ iDB ] );
}

/*** DBMoveRecord ************************************************************
*
*   Purpose:   Moves a record from one index to another
*
*   Returns:   Nothing
*
*   Created:   4/21/1997  Eric Kenslow
*****************************************************************************/

void DBMoveRecord( DBIndexType iDB, UInt iFrom, UInt iTo )
{
  if( iTo > iFrom )
    DmMoveRecord( mg_hDB[ iDB ], iFrom, iTo + 1 );
  else
    DmMoveRecord( mg_hDB[ iDB ], iFrom, iTo );
}

