/*
 * LaunchPad 1.11 public code release - October 1, 1997
 *  Official LaunchPad page: http://www.nwlink.com/~emilyk/LaunchPad
 *
 *  This code compiles with the EXCELLENT GCC 0.4.0 Win32 port for PalmPilot.
 *  Find info about it at http://www.shoppersmart.com/jlehett/gccwin32.html
 *  Does not compile under the 0.5.0 port because of PalmOS 2.0 differences.
 *
 * Disclaimers and other legal stuff:
 *  This code is provided AS-IS; there is NO WARRANTY, express or implied.
 *  The user assumes full responsibility and liability for any use of
 *  this code.
 *
 *  Copyright (C)1997 Eric Kenslow.
 *  All Rights Reserved by Eric Kenslow, except as noted below.
 *
 *  Permission to distribute this package complete and unmodified
 *  for personal use via the internet for no charge (other than reasonable
 *  connection charges) is granted to the public.
 *  Distribution for commercial purposes via any other medium, including but
 *  not limited to CD-ROM, floppy, or printed material, in whole or in part,
 *  requires express permission from Eric Kenslow (erick@metainfo.com).
 *
 *  Modified versions may be distributed with this notice intact, provided
 *  that said versions are clearly marked as modified code. The above
 *  provisions also apply to any such modified packages.
 *
 *  Permission is also granted to the public to use PARTS of this code in
 *  other projects, with no special notice required unless the source code
 *  is part of the distribution, in which case this notice must remain
 *  conspicuous in the code that was used.
 *
 * Note: These terms are intended to protect me, not make this package less
 *       useful. If you have a special need or use for this package, contact
 *       me and we'll work something out.
 *                                                - Eric
 *                                                  erick@metainfo.com
 *
 */


/*****************************************************************************
*
*   Project name: LaunchPad
*   Owner:        Eric Kenslow
*   Date:         3/8/1997
*
*   Synopsis:
*     App launcher
*
*****************************************************************************/

#include "LaunchPad.h"
#include "Res.h"
#include <featuremgr.h>

// Global variables

volatile  LauncherPrefsType g_prefs;

// End global variables

// Modular global functions

static  DWord StartApplication  ( void );
static  void  EventLoop         ( void );
static  void  StopApplication   ( void );

// End modular globals

/*** PilotMain ***************************************************************
*
*   Purpose:   Main program flow control
*
*   Returns:   0 on success
*
*   Created:   3/8/1997  Eric Kenslow
*****************************************************************************/

DWord PilotMain( Word cmd, Ptr cmdPBP, Word launchFlags )
{
  DWord err = 0;

  switch( cmd )
    {
    case sysAppLaunchCmdSyncNotify:
      {
      // Set to refresh the next time we're launched
      FtrSet( g_dwCreatorID, g_iFtrRefresh, true );
      break;
      }

    case sysAppLaunchCmdSystemReset:
      {
      // Re-hook the apps button
      LauncherPrefsType prefs;
      if( PrefGetAppPreferences( g_dwCreatorID, g_dwVersion,
                                  &prefs, sizeof( prefs ) ) &&
          prefs.flags.fDefaultLauncher )
        PrefsHookApps();
      // Set to refresh the next time we're launched
      FtrSet( g_dwCreatorID, g_iFtrRefresh, true );
      break;
      }

    case sysAppLaunchCmdNormalLaunch:
      {
      err = StartApplication();
      if( !err )
        {
        // Open our main form
        FrmGotoForm( idFormMain );
        EventLoop();
        StopApplication();
        }
      else
        {
        FrmAlert( idAlertErrInit );
        SysReset();
        }
      break;
      }

    default:
      {
      err = sysErrParamErr;
      break;
      }
    }
  return err;
}

/*** StartApplication ********************************************************
*
*   Purpose:   Performs one-time global app initialization
*
*   Returns:   0 on success
*
*   Created:   3/8/1997  Eric Kenslow
*****************************************************************************/

DWord StartApplication( void )
{
  DWord err = 0;

#ifdef _DEBUG
  MemSetDebugMode( memDebugModeCheckOnChange |
                   memDebugModeScrambleOnChange |
                   memDebugModeFillFree );
#else
  MemSetDebugMode( 0 );
#endif

  // Load our prefs
  if( !PrefGetAppPreferences( g_dwCreatorID, g_dwVersion,
                               &g_prefs, sizeof( g_prefs ) ) )
    {
    // Default initialization
    MemSet( &g_prefs, sizeof( g_prefs ), 0 );

    g_prefs.flags.fDragKeepsTab  = true;
    g_prefs.flags.fStartOnTab0   = false;

    g_prefs.flags.fShowDate      = timeTime;
    g_prefs.flags.fShowBar       = voltVolt;

    g_prefs.data.cAppColumns = 3;

    g_prefs.maskGadgets = (1 << 0) | (1 << 1) | (1 << 2);

    // First run; show the help, if desired
    if( FrmAlert( idAlertFirstRun ) == 0 )
      FrmHelp( idFormMain );
    }
  if( g_prefs.flags.fStartOnTab0 )
    g_prefs.iTabCur = 0;
  if( g_prefs.data.cAppColumns == 0 ) // Upgrade?
    g_prefs.data.cAppColumns = 3;
  if( g_prefs.data.cSecCycleTime == 0 )
    {
    g_prefs.data.cSecCycleTime = 2;
    g_prefs.data.cSecCycleVolt = 2;
    }
  if( g_prefs.flags.fGadgetsUnused && g_prefs.maskGadgets == 0 )
    g_prefs.maskGadgets = (1 << 0) | (1 << 1) | (1 << 2);

  // Finish initialization
  if( !DBInit() )
    {
    err = -1;
    }

  return err;
}

/*** StopApplication *********************************************************
*
*   Purpose:   Performs one-time global app cleanup
*
*   Returns:   Nothing
*
*   Created:   3/8/1997  Eric Kenslow
*****************************************************************************/

void StopApplication( void )
{
  WinManCleanup();
  DBCleanup();
  PrefSetAppPreferences( g_dwCreatorID, g_dwVersion,
                          &g_prefs, sizeof( g_prefs ) );

#ifdef _DEBUG
  MemSetDebugMode( 0 );
#endif
}

/*** EventLoop ***************************************************************
*
*   Purpose:   Pumps the global app message queue
*
*   Returns:   Nothing
*
*   Created:   3/8/1997  Eric Kenslow
*****************************************************************************/

void EventLoop( void )
{
  static  EventType evt;

  do {
    Word    err;

    EvtGetEvent( &evt, sysTicksPerSecond );

    if( !SysHandleEvent( &evt ) &&
        !MenuHandleEvent( MenuGetActiveMenu(), &evt, &err ) )
      {
      switch( evt.eType )
        {
        case frmLoadEvent:
          {
          FormPtr pfrm = FrmInitForm( evt.data.frmLoad.formID );
          FrmSetActiveForm( pfrm );

          switch( evt.data.frmLoad.formID )
            {
            case idFormMain:
              FrmSetEventHandler( pfrm, MainFormHandler );
              break;

            case idFormRename:
              FrmSetEventHandler( pfrm, RenameFormHandler );
              break;

            case idFormNewTab:
              FrmSetEventHandler( pfrm, NewTabFormHandler );
              break;

            case idFormPrefs:
              FrmSetEventHandler( pfrm, PrefsFormHandler );
              break;

            case idFormAbout:
              FrmSetEventHandler( pfrm, AboutFormHandler );
              break;

            default:
              {
#ifdef _DEBUG
              static  char  sz[] = "Unknown Form ID";
              ErrDisplay( sz );
#endif // DEBUG
              break;
              }
            }
          }
          break;

        default:
          FrmDispatchEvent( &evt );
          break;
        }
      }
    } while( evt.eType != appStopEvent );
}

