/*
 * LaunchPad 1.11 public code release - October 1, 1997
 *  Official LaunchPad page: http://www.nwlink.com/~emilyk/LaunchPad
 *
 *  This code compiles with the EXCELLENT GCC 0.4.0 Win32 port for PalmPilot.
 *  Find info about it at http://www.shoppersmart.com/jlehett/gccwin32.html
 *  Does not compile under the 0.5.0 port because of PalmOS 2.0 differences.
 *
 * Disclaimers and other legal stuff:
 *  This code is provided AS-IS; there is NO WARRANTY, express or implied.
 *  The user assumes full responsibility and liability for any use of
 *  this code.
 *
 *  Copyright (C)1997 Eric Kenslow.
 *  All Rights Reserved by Eric Kenslow, except as noted below.
 *
 *  Permission to distribute this package complete and unmodified
 *  for personal use via the internet for no charge (other than reasonable
 *  connection charges) is granted to the public.
 *  Distribution for commercial purposes via any other medium, including but
 *  not limited to CD-ROM, floppy, or printed material, in whole or in part,
 *  requires express permission from Eric Kenslow (erick@metainfo.com).
 *
 *  Modified versions may be distributed with this notice intact, provided
 *  that said versions are clearly marked as modified code. The above
 *  provisions also apply to any such modified packages.
 *
 *  Permission is also granted to the public to use PARTS of this code in
 *  other projects, with no special notice required unless the source code
 *  is part of the distribution, in which case this notice must remain
 *  conspicuous in the code that was used.
 *
 * Note: These terms are intended to protect me, not make this package less
 *       useful. If you have a special need or use for this package, contact
 *       me and we'll work something out.
 *                                                - Eric
 *                                                  erick@metainfo.com
 *
 */


/*****************************************************************************
*
*   Module name:  MainForm
*   Project:      LaunchPad
*
*   Created:      3/8/1997  Eric Kenslow
*
*   Description:  Main form code & data
*
*****************************************************************************/

#include "LaunchPad.h"
#include "Res.h"
#include <FeatureMgr.h>

// Modular global variables

static  TimeFormatType  mg_timeFormat;
static  DateFormatType  mg_dateFormat;

// End modular globals

// Modular macros

#define CLICK() \
        (SndPlaySystemSound( sndClick ))

#define ERROR() \
        (SndPlaySystemSound( sndError ))

// End modular macros


/*** UpdateScrollers *********************************************************
*
*   Purpose:   Updates the main form's scrollers and gadgets
*
*   Returns:   Nothing
*
*   Created:   4/12/1997  Eric Kenslow
*****************************************************************************/

static void UpdateScrollers( void )
{
  FormPtr pFrm = FrmGetFormPtr( idFormMain );
  ControlPtr pLt  = FrmGetObjectPtr( pFrm,
                                      FrmGetObjectIndex( pFrm, idScrollTabsLt ) );
  ControlPtr pRt  = FrmGetObjectPtr( pFrm,
                                      FrmGetObjectIndex( pFrm, idScrollTabsRt ) );

  FrmUpdateScrollers( pFrm, FrmGetObjectIndex( pFrm, idScrollUp ),
                            FrmGetObjectIndex( pFrm, idScrollDn ),
                            AppManCanScroll( up ), AppManCanScroll( down ) );
  if( TabManCanScroll( right ) )
    CtlShowControl( pRt );
  else
    CtlHideControl( pRt );
  if( TabManCanScroll( left ) )
    CtlShowControl( pLt );
  else
    CtlHideControl( pLt );
}

/*** DrawGadgets *************************************************************
*
*   Purpose:   Draws our gadgets
*
*   Returns:   Nothing
*
*   Created:   4/18/1997  Eric Kenslow
*****************************************************************************/

static void DrawGadgets( void )
{
  FormPtr pFrm = FrmGetFormPtr( idFormMain );
  UInt    rgIDGadgets[] = { idPilotMemory, idGadOff, idGadLock };
  UInt    i;
  for( i = 0; i < sizeof( rgIDGadgets ) / sizeof( rgIDGadgets[ 0 ] ); i++ )
    {
    RectangleType rc;
    UInt iGad = FrmGetObjectIndex( pFrm, rgIDGadgets[ i ] );
    FrmGetObjectBounds( pFrm, iGad, &rc );

    if( g_prefs.maskGadgets & (1 << i) )
      {
      VoidHand  hBmp;
      CtlShowControl( FrmGetObjectPtr( pFrm, iGad ) );
      hBmp = DmGetResource( (ULong)'Tbmp', rgIDGadgets[ i ] );
      if( hBmp )
        {
        WinDrawBitmap( MemHandleLock( hBmp ), rc.topLeft.x, rc.topLeft.y );
        MemHandleUnlock( hBmp );
        DmReleaseResource( hBmp );
        }
      }
    else
      {
      CtlHideControl( FrmGetObjectPtr( pFrm, iGad ) );
      WinEraseRectangle( &rc, 0 );
      }
    }
}

/*** LaunchBuiltinApp ********************************************************
*
*   Purpose:   Launches a built-in pilot app
*
*   Returns:   Nothing (does not return)
*
*   Created:   4/22/1997  Eric Kenslow
*****************************************************************************/

static void LaunchBuiltinApp( ULong idApp )
{
  DmSearchStateType sst;
  UInt              iCardDB;
  LocalID           idDB;
  DmGetNextDatabaseByTypeCreator( true, &sst, (ULong)'appl',
                                   idApp, true, &iCardDB, &idDB );
  SysUIAppSwitch( iCardDB, idDB, sysAppLaunchCmdNormalLaunch, 0 );
}

/*** AddNilEventToQueue ******************************************************
*
*   Purpose:   Figure it out for yourself!
*
*   Returns:   Nothing
*
*   Created:   4/21/1997  Eric Kenslow
*****************************************************************************/

static void AddNilEventToQueue( void )
{
  EventType eT;
  MemSet( &eT, sizeof( eT ), 0 );
  eT.eType = nilEvent;
  EvtAddEventToQueue( &eT );
}

/*** MainFormHandler *********************************************************
*
*   Purpose:   Event handler for the main form
*
*   Returns:   true if event was fully processed
*
*   Created:   3/8/1997  Eric Kenslow
*****************************************************************************/

Boolean MainFormHandler( EventPtr pevt )
{
  Boolean fHandled = true;
  Boolean fModified = false;
  FormPtr pMe = FrmGetActiveForm();
  static  Byte  iTabLast;
  static  ULong cSecsTimePrev, cSecsVoltPrev;

  switch( pevt->eType )
    {
    case frmOpenEvent:
      {
      SystemPreferencesType prefSys;

      iTabLast = (Byte)-1;

      PrefGetPreferences( &prefSys );
      mg_timeFormat = prefSys.timeFormat;
      mg_dateFormat = prefSys.dateFormat;

      cSecsVoltPrev = TimGetTicks() / sysTicksPerSecond;
      cSecsTimePrev = cSecsVoltPrev;

      if( !WinManInit() )
        {
        FrmAlert( idAlertErrInit );
        SysReset();
        }
      fModified = true;
      break;
      }

    case frmUpdateEvent:
      {
      UpdateScrollers();
      FrmDrawForm( pMe );
      TabManDrawTabs();
      AppManDrawApps();
      DrawGadgets();
      break;
      }

    case penDownEvent:
      {
      if( !AppManHitTest( pevt->screenX, pevt->screenY, true ) )
        {
        iTabLast = TabManHitTest( pevt->screenX, pevt->screenY );
        if( iTabLast == (Byte)-1 )
          fHandled = false;
        }
      break;
      }

    case penMoveEvent:
      {
      AppManDrag( pevt->screenX, pevt->screenY );
      TabManDrag( pevt->screenX, pevt->screenY );
      fHandled = false;
      break;
      }

    case penUpEvent:
      {
      // Make sure there's not a menu or something up
      if( (FormPtr)WinGetActiveWindow() == pMe )
        {
        Byte  iTabNew = TabManHitTest( pevt->screenX, pevt->screenY );
        AppManDrag( -1, -1 );
        TabManDrag( -1, -1 );
        if( iTabNew != (Byte)-1 )
          {
          CLICK();
          if( iTabLast != (Byte)-1 && iTabLast != iTabNew )
            {
            AppManMoveTab( iTabLast, iTabNew );
            TabManMoveTab( iTabLast, iTabNew );
            g_prefs.iTabCur = iTabNew;
            }
          else if( !AppManMoveLast( iTabNew ) || g_prefs.flags.fDragKeepsTab )
            g_prefs.iTabCur = iTabNew;
          fModified = true;
          }
        else if( AppManHitTest( pevt->screenX, pevt->screenY, false ) )
          {
          CLICK();
          AppManLaunchLast();
          }
        else
          fHandled = false;
        }
      else
        fHandled = false;
      iTabLast = -1;
      break;
      }

    case menuEvent:
    case ctlSelectEvent:
      {
      EventType eT;
      MemSet( &eT, sizeof( eT ), 0 );
      eT.eType = keyDownEvent;

      switch( pevt->data.ctlSelect.controlID )
        {
        // Menu events

        case idInfoAbout:
          FrmPopupForm( idFormAbout );
          break;

        case idInfoHelp:
          FrmHelp( idFormMain );
          break;

        case idTabNew:
          FrmPopupForm( idFormNewTab );
          break;

        case idTabRename:
          FrmPopupForm( idFormRename );
          break;

        case idTabDelete:
          if( FrmAlert( idAlertConfirmDelete ) )
            {
            if( TabManDelete() )
              {
              AppManClearTab();
              g_prefs.iTabCur = 0;
              fModified = true;
              }
            else
              FrmAlert( idAlertErrDelete );
            }
          break;

        case idAppPrefs:
          FrmPopupForm( idFormPrefs );
          break;

        case idOptionsReset:
          SysReset();
          break;

        // Control events

        case idScrollUp:
          eT.data.keyDown.chr = pageUpChr;
          EvtAddEventToQueue( &eT );
          break;

        case idScrollDn:
          eT.data.keyDown.chr = pageDownChr;
          EvtAddEventToQueue( &eT );
          break;

        case idScrollTabsRt:
          if( TabManCanScroll( right ) )
            {
            TabManScroll( right );
            fModified = true;
            }
          else
            ERROR();
          break;

        case idScrollTabsLt:
          if( TabManCanScroll( left ) )
            {
            TabManScroll( left );
            fModified = true;
            }
          else
            ERROR();
          break;

        case idLabelTime:
          g_prefs.flags.fShowDate = (g_prefs.flags.fShowDate + 1) % timeMax;
          AddNilEventToQueue();
          break;

        case idLabelVolt:
          g_prefs.flags.fShowBar = (g_prefs.flags.fShowBar + 1) % voltMax;
          AddNilEventToQueue();
          break;

        case idGadLock:
          {
          SystemPreferencesType prefs;

          // First, see what version we're running on
          DWord dwROMVer = 0;
          FtrGet( sysFtrCreator, sysFtrNumROMVersion, &dwROMVer );
          ErrFatalDisplayIf( !dwROMVer, "OH NO!" );

          PrefGetPreferences( &prefs );

          // We're only interested in the major version, which is
          //  stored in the high byte
          switch( dwROMVer >> 24 )
            {
            case 1:
              {
              // Good ol' version 1 ROM
              prefs.deviceLocked      = true;
              prefs.hideSecretRecords = true;
              break;
              }

            case 2:
              {
              // TODO: Update to 2.0 SDK
              *(((Byte*)&(prefs.deviceLocked)) + 1)      = true;
              *(((Byte*)&(prefs.hideSecretRecords)) + 1) = true;
              break;
              }

            default:
              {
              // Electra or Communica?
              SndPlaySystemSound( sndError );
              return false;
              }
            }
          PrefSetPreferences( &prefs );
          SysSleep( false, false );
          LaunchBuiltinApp( (ULong)'secr' );

          break;
          }

        case idGadOff:
          SysSleep( false, false );
          break;

        case idPilotMemory:
          {
          LaunchBuiltinApp( (ULong)'memr' );
          break;
          }

        case idPilotPrefs:
          {
          LaunchBuiltinApp( (ULong)'pref' );
          break;
          }

        case idPilotSecurity:
          {
          LaunchBuiltinApp( (ULong)'secr' );
          break;
          }

        case idPilotHotSync:
          {
          LaunchBuiltinApp( (ULong)'sync' );
          break;
          }

        default:
          fHandled = false;
          break;
        }
      break;
      }

    case keyDownEvent:
      {
      switch( pevt->data.keyDown.chr )
        {
        case pageUpChr:
          {
          if( !AppManScroll( up ) )
            ERROR();
          else
            fModified = true;
          break;
          }
        case pageDownChr:
          {
          if( !AppManScroll( down ) )
            ERROR();
          else
            fModified = true;
          break;
          }
        default:
          fHandled = false;
          break;
        }
      break;
      }

    case nextTabEvent:
      {
      TabManNextTab();
      fModified = true;
      break;
      }

    case nilEvent:
      {
      // Update the time & voltage displays
      FormPtr pMe = FrmGetFormPtr( idFormMain );
      Word    iTime = FrmGetObjectIndex( pMe, idLabelTime );
      Word    iVolt = FrmGetObjectIndex( pMe, idLabelVolt );
      char    szBuf[ longDateStrLenght ];  // sic
      UInt    iVolts, iWarn;
      DateTimeType  time;

      ULong   cSecsCur = TimGetTicks() / sysTicksPerSecond;
      if( g_prefs.flags.fCycleTime &&
          cSecsCur - cSecsTimePrev >= (ULong)g_prefs.data.cSecCycleTime )
        {
        EventType eT;
        eT.eType = ctlSelectEvent;
        eT.data.ctlSelect.controlID = idLabelTime;
        EvtAddEventToQueue( &eT );
        cSecsTimePrev = TimGetTicks() / sysTicksPerSecond;
        }
      if( g_prefs.flags.fCycleVolt &&
          cSecsCur - cSecsVoltPrev >= (ULong)g_prefs.data.cSecCycleVolt )
        {
        EventType eT;
        eT.eType = ctlSelectEvent;
        eT.data.ctlSelect.controlID = idLabelVolt;
        EvtAddEventToQueue( &eT );
        cSecsVoltPrev = TimGetTicks() / sysTicksPerSecond;
        }

      TimSecondsToDateTime( TimGetSeconds(), &time );

      switch( g_prefs.flags.fShowDate )
        {
        case timeTime:
          TimeToAscii( time.hour, time.minute, mg_timeFormat, szBuf );
          break;
        case timeDate:
          DateToAscii( time.month, time.day, time.year, mg_dateFormat,
                        szBuf );
          break;
        case timeDayOfWeek:
          SysCopyStringResource( szBuf,
                              idStrSunday +
                               DayOfWeek( time.month, time.day, time.year ) );
          break;
        }
      FrmHideObject( pMe, iTime );
      FrmCopyLabel( pMe, idLabelTime, szBuf );
      FrmShowObject( pMe, iTime );

      iVolts = SysBatteryInfo( false, &iWarn, 0, 0, 0, 0 );
      switch( g_prefs.flags.fShowBar )
        {
        case voltVolt:
          {
          FrmHideObject( pMe, iVolt );
          StrIToA( szBuf, iVolts / 100 );
          if( iVolts % 100 < 10 )
            StrCat( szBuf, ".0" );
          else
            StrCat( szBuf, "." );
          StrIToA( &(szBuf[ StrLen( szBuf ) ]), iVolts % 100 );
          StrCat( szBuf, "v" );
          FrmCopyLabel( pMe, idLabelVolt, szBuf );
          FrmShowObject( pMe, iVolt );
          break;
          }

        case voltBar:
          {
          double  rVoltPct;
          RectangleType rc, rcText;
          FrmGetObjectBounds( pMe, iVolt, &rc );
          RctCopyRectangle( &rc, &rcText );

          szBuf[ 0 ] = '\0';
          FrmHideObject( pMe, iVolt );
          FrmCopyLabel( pMe, idLabelVolt, szBuf );
          FrmShowObject( pMe, iVolt );

          RctInsetRectangle( &rc, 1 );
          WinDrawRectangleFrame( simpleFrame, &rc );
          WinSetClip( &rc );

          // Calculate how much of the bar to draw
          rVoltPct = ((double)(iVolts - iWarn) / (double)(300 - iWarn));
          rc.extent.x *= rVoltPct;
          WinDrawRectangle( &rc, 0 );
          StrIToA( szBuf, (int)(rVoltPct * 100.0) );
          StrCat( szBuf, "%" );
          FntSetFont( 0 );
          WinInvertChars( szBuf, StrLen( szBuf ),
                          rcText.topLeft.x + (rcText.extent.x / 2) -
                           (FntCharsWidth( szBuf, StrLen( szBuf ) ) / 2),
                          rcText.topLeft.y - 1 );
          WinResetClip();
          break;
          }
        }
      break;
      }

    default:
      fHandled = false;
      break;
    }

  if( fModified )
    {
    FrmUpdateForm( FrmGetActiveFormID(), 0 );
    // Shove a nil event so the gadgets are updated
    AddNilEventToQueue();
    }

  return fHandled;
}

