/*
 * LaunchPad 1.11 public code release - October 1, 1997
 *  Official LaunchPad page: http://www.nwlink.com/~emilyk/LaunchPad
 *
 *  This code compiles with the EXCELLENT GCC 0.4.0 Win32 port for PalmPilot.
 *  Find info about it at http://www.shoppersmart.com/jlehett/gccwin32.html
 *  Does not compile under the 0.5.0 port because of PalmOS 2.0 differences.
 *
 * Disclaimers and other legal stuff:
 *  This code is provided AS-IS; there is NO WARRANTY, express or implied.
 *  The user assumes full responsibility and liability for any use of
 *  this code.
 *
 *  Copyright (C)1997 Eric Kenslow.
 *  All Rights Reserved by Eric Kenslow, except as noted below.
 *
 *  Permission to distribute this package complete and unmodified
 *  for personal use via the internet for no charge (other than reasonable
 *  connection charges) is granted to the public.
 *  Distribution for commercial purposes via any other medium, including but
 *  not limited to CD-ROM, floppy, or printed material, in whole or in part,
 *  requires express permission from Eric Kenslow (erick@metainfo.com).
 *
 *  Modified versions may be distributed with this notice intact, provided
 *  that said versions are clearly marked as modified code. The above
 *  provisions also apply to any such modified packages.
 *
 *  Permission is also granted to the public to use PARTS of this code in
 *  other projects, with no special notice required unless the source code
 *  is part of the distribution, in which case this notice must remain
 *  conspicuous in the code that was used.
 *
 * Note: These terms are intended to protect me, not make this package less
 *       useful. If you have a special need or use for this package, contact
 *       me and we'll work something out.
 *                                                - Eric
 *                                                  erick@metainfo.com
 *
 */


/*****************************************************************************
*
*   Module name:  Prefs
*   Project:      LaunchPad
*
*   Created:      4/15/1997  Eric Kenslow
*
*   Description:  Handler for the preferences form
*
*****************************************************************************/

#include "LaunchPad.h"
#include <FeatureMgr.h>
#include "res.h"

// Modular global data

static  Boolean mg_fIsDef;

// End modular global data

/*** PrefsHookApps ***********************************************************
*
*   Purpose:   Hooks the apps button
*
*   Returns:   Nothing
*
*   Created:   4/26/1997  Eric Kenslow
*****************************************************************************/

void PrefsHookApps( void )
{
  VoidHand  hCode = DmGetResource( (ULong)'code', idCodeHack );
  if( hCode )
    {
    VoidPtr pCode = MemHandleLock( hCode );
    FtrSet( g_dwCreatorID, g_iFtrOldTrap,
             (DWord)SysGetTrapAddress( sysTrapSysAppLauncherDialog ) );
    SysSetTrapAddress( sysTrapSysAppLauncherDialog, pCode );
    DmReleaseResource( hCode );
    }
}

/*** UpdateControls **********************************************************
*
*   Purpose:   Updates the preference controls
*
*   Returns:   Nothing
*
*   Created:   4/15/1997  Eric Kenslow
*****************************************************************************/

static void UpdateControls( FormPtr pfrm, LauncherPrefsPtr pprefs )
{
  char    szT[ 32 ];
  DWord   dw;
  if( FtrGet( g_dwCreatorID, g_iFtrOldTrap, &dw ) != 0 )
    mg_fIsDef = false;
  else
    mg_fIsDef = !(dw ==
                  (DWord)SysGetTrapAddress( sysTrapSysAppLauncherDialog ));

  FrmSetControlValue( pfrm,
                       FrmGetObjectIndex( pfrm, idCheckDefLauncher ),
                       (short)mg_fIsDef );
  FrmSetControlValue( pfrm,
                       FrmGetObjectIndex( pfrm, idCheckKeepTab ),
                       pprefs->flags.fDragKeepsTab );
  FrmSetControlValue( pfrm,
                       FrmGetObjectIndex( pfrm, idCheckStartOnTab0 ),
                       pprefs->flags.fStartOnTab0 );
  FrmSetControlValue( pfrm,
                       FrmGetObjectIndex( pfrm, idCheckPageScroll ),
                       pprefs->flags.fPageScroll );
  FrmSetControlValue( pfrm,
                       FrmGetObjectIndex( pfrm, idCheckUseMemory ),
                       pprefs->maskGadgets & (1 << 0) );
  FrmSetControlValue( pfrm,
                       FrmGetObjectIndex( pfrm, idCheckUseOff ),
                       pprefs->maskGadgets & (1 << 1) );
  FrmSetControlValue( pfrm,
                       FrmGetObjectIndex( pfrm, idCheckUseLock ),
                       pprefs->maskGadgets & (1 << 2) );

  FrmSetControlGroupSelection( pfrm, idGroupColumns,
                       ((Word)pprefs->data.cAppColumns) - 2 + idColumns2 );

  FrmSetControlValue( pfrm,
                       FrmGetObjectIndex( pfrm, idCheckCycleTime ),
                       pprefs->flags.fCycleTime );
  StrIToA( szT, pprefs->data.cSecCycleTime );
  FrmHideObject( pfrm, FrmGetObjectIndex( pfrm, idPopupTimeSeconds ) );
  FrmCopyLabel( pfrm, idPopupTimeSeconds, szT );
  FrmShowObject( pfrm, FrmGetObjectIndex( pfrm, idPopupTimeSeconds ) );

  FrmSetControlValue( pfrm,
                       FrmGetObjectIndex( pfrm, idCheckCycleVolt ),
                       pprefs->flags.fCycleVolt );
  StrIToA( szT, pprefs->data.cSecCycleVolt );
  FrmHideObject( pfrm, FrmGetObjectIndex( pfrm, idPopupVoltSeconds ) );
  FrmCopyLabel( pfrm, idPopupVoltSeconds, szT );
  FrmShowObject( pfrm, FrmGetObjectIndex( pfrm, idPopupVoltSeconds ) );
}

/*** PrefsFormHandler ********************************************************
*
*   Purpose:   Handles events for the prefs form
*
*   Returns:   true if the event was fully handled
*
*   Created:   4/15/1997  Eric Kenslow
*****************************************************************************/

Boolean PrefsFormHandler( EventPtr pevt )
{
  Boolean fHandled = true;
  FormPtr pMe = FrmGetActiveForm();
  static  LauncherPrefsType prefsT;

  switch( pevt->eType )
    {
    case frmOpenEvent:
      {
      MemMove( &prefsT, &g_prefs, sizeof( prefsT ) );
      UpdateControls( pMe, &prefsT );

      FrmDrawForm( pMe );
      break;
      }

    case ctlSelectEvent:
      {
      Boolean fUpdate = true;
      switch( pevt->data.ctlSelect.controlID )
        {
        case idCheckDefLauncher:
          {
          if( mg_fIsDef )
            {
            VoidPtr p = SysGetTrapAddress( sysTrapSysAppLauncherDialog );
            VoidPtr pOld;
            FtrGet( g_dwCreatorID, g_iFtrOldTrap, (DWordPtr)&pOld );
            SysSetTrapAddress( sysTrapSysAppLauncherDialog, pOld );
            FtrUnregister( g_dwCreatorID, g_iFtrOldTrap );
            if( p != pOld )
              MemPtrUnlock( p );
            prefsT.flags.fDefaultLauncher = false;
            g_prefs.flags.fDefaultLauncher = false;
            }
          else
            {
            PrefsHookApps();
            prefsT.flags.fDefaultLauncher = true;
            g_prefs.flags.fDefaultLauncher = true;
            }
          break;
          }

        case idColumns2:
        case idColumns3:
        case idColumns4:
        case idColumns5:
          prefsT.data.cAppColumns = (pevt->data.ctlSelect.controlID -
                                     idColumns2) + 2;
          break;

        case idCheckCycleTime:
          prefsT.flags.fCycleTime = !(prefsT.flags.fCycleTime);
          break;
        case idCheckCycleVolt:
          prefsT.flags.fCycleVolt = !(prefsT.flags.fCycleVolt);
          break;

        case idPopupTimeSeconds:
        case idPopupVoltSeconds:
          {
          FormPtr     pMe = FrmGetActiveForm();
          ListPtr     plst = FrmGetObjectPtr( pMe,
                               FrmGetObjectIndex( pMe, idLstSeconds ) );
          Word        iSelect;
          if( pevt->data.ctlSelect.controlID == idPopupTimeSeconds )
            LstSetSelection( plst, prefsT.data.cSecCycleTime - 1 );
          else
            LstSetSelection( plst, prefsT.data.cSecCycleVolt - 1 );
          LstPopupList( plst );
          iSelect = LstGetSelection( plst );
          if( iSelect != noListSelection )
            {
            if( pevt->data.ctlSelect.controlID == idPopupTimeSeconds )
              {
              prefsT.flags.fCycleTime = true;
              prefsT.data.cSecCycleTime = iSelect + 1;
              }
            else
              {
              prefsT.flags.fCycleVolt = true;
              prefsT.data.cSecCycleVolt = iSelect + 1;
              }
            }
          break;
          }

        case idCheckKeepTab:
          prefsT.flags.fDragKeepsTab = !(prefsT.flags.fDragKeepsTab);
          break;

        case idCheckStartOnTab0:
          prefsT.flags.fStartOnTab0 = !(prefsT.flags.fStartOnTab0);
          break;

        case idCheckPageScroll:
          prefsT.flags.fPageScroll = !(prefsT.flags.fPageScroll);
          break;

        case idCheckUseMemory: case idCheckUseOff: case idCheckUseLock:
          {
          Byte  mask = 1 <<
                          (pevt->data.ctlSelect.controlID - idCheckUseMemory);
          if( prefsT.maskGadgets & mask )
            prefsT.maskGadgets &= ~mask;
          else
            prefsT.maskGadgets |= mask;
          break;
          }

        case idOK:
          MemMove( &g_prefs, &prefsT, sizeof( prefsT ) );
          AppManRecalcRects();
          FrmUpdateForm( idFormMain, 0 );
          // Fall through
        case idCancel:
          FrmReturnToForm( 0 );
          fUpdate = false;
          PrefSetAppPreferences( g_dwCreatorID, g_dwVersion,
                                  &g_prefs, sizeof( g_prefs ) );
          break;
        }
      if( fUpdate )
        UpdateControls( pMe, &prefsT );
      break;
      }

    default:
      fHandled = false;
      break;
    }

  return fHandled;
}

