/*
 * LaunchPad 1.11 public code release - October 1, 1997
 *  Official LaunchPad page: http://www.nwlink.com/~emilyk/LaunchPad
 *
 *  This code compiles with the EXCELLENT GCC 0.4.0 Win32 port for PalmPilot.
 *  Find info about it at http://www.shoppersmart.com/jlehett/gccwin32.html
 *  Does not compile under the 0.5.0 port because of PalmOS 2.0 differences.
 *
 * Disclaimers and other legal stuff:
 *  This code is provided AS-IS; there is NO WARRANTY, express or implied.
 *  The user assumes full responsibility and liability for any use of
 *  this code.
 *
 *  Copyright (C)1997 Eric Kenslow.
 *  All Rights Reserved by Eric Kenslow, except as noted below.
 *
 *  Permission to distribute this package complete and unmodified
 *  for personal use via the internet for no charge (other than reasonable
 *  connection charges) is granted to the public.
 *  Distribution for commercial purposes via any other medium, including but
 *  not limited to CD-ROM, floppy, or printed material, in whole or in part,
 *  requires express permission from Eric Kenslow (erick@metainfo.com).
 *
 *  Modified versions may be distributed with this notice intact, provided
 *  that said versions are clearly marked as modified code. The above
 *  provisions also apply to any such modified packages.
 *
 *  Permission is also granted to the public to use PARTS of this code in
 *  other projects, with no special notice required unless the source code
 *  is part of the distribution, in which case this notice must remain
 *  conspicuous in the code that was used.
 *
 * Note: These terms are intended to protect me, not make this package less
 *       useful. If you have a special need or use for this package, contact
 *       me and we'll work something out.
 *                                                - Eric
 *                                                  erick@metainfo.com
 *
 */


/*****************************************************************************
*
*   Module name:  TabMan
*   Project:      LaunchPad
*
*   Created:      3/30/1997  Eric Kenslow
*
*   Description:  Code & data for the tab manager
*
*****************************************************************************/

#include "LaunchPad.h"
#include "res.h"

// Modular constants

#define dxText  ((SWord)1)
#define dyText  ((SWord)1)

// End modular constants

// Modular types

typedef struct tagTab
{
  char  szTitle[ g_cbmaxTabName ];
} TabInfoType, *TabInfoPtr;

typedef struct tagTabItem
{
  struct tagTabItem*  pNext;
  RectangleType rc;
  TabInfoPtr    pInfo;
} TabItemType, *TabItemPtr;

// End modular types

// Modular globals

static  RectangleType mg_rcTabs, mg_rcTabsScreen;
static  RectangleType mg_rcDrag;
static  TabItemPtr    mg_pTabHead;
static  WinHandle     mg_hwinTabs;

// End modular globals

// Modular functions

static  void    CalcTabRects  ( void );

// End modular functions

/*** AddTab ******************************************************************
*
*   Purpose:   Does the internal work of adding a tab
*
*   Returns:   true if successful
*
*   Created:   3/30/1997  Eric Kenslow
*****************************************************************************/

static Boolean AddTab( TabInfoPtr pInfo )
{
  TabItemPtr  pitem = MemPtrNew( sizeof( TabItemType ) );
  if( pitem )
    {
    TabItemPtr  pCur = mg_pTabHead;
    MemSet( pitem, sizeof( TabItemType ), 0 );
    pitem->pInfo = pInfo;

    ErrFatalDisplayIf( !pInfo, "Attempt to add NULL info" );

    while( pCur && pCur->pNext )
      pCur = pCur->pNext;

    if( pCur )
      pCur->pNext = pitem;
    else
      mg_pTabHead = pitem;

    CalcTabRects();
    return true;
    }
  return false;
}

/*** InternalScroll **********************************************************
*
*   Purpose:   Scrolls
*
*   Returns:   Nothing
*
*   Created:   4/26/1997  Eric Kenslow
*****************************************************************************/

static void InternalScroll( int dx, Boolean fUpdateGlobal )
{
  TabItemPtr  pCur = mg_pTabHead;
  while( pCur )
    {
    RctOffsetRectangle( &(pCur->rc), dx, 0 );
    pCur = pCur->pNext;
    }
  if( fUpdateGlobal )
    g_prefs.dxTabScroll += dx;
}

/*** CalcTabRects ************************************************************
*
*   Purpose:   Calculates the rectangles for all tabs
*
*   Returns:   Nothing
*
*   Created:   3/30/1997  Eric Kenslow
*****************************************************************************/

void CalcTabRects( void )
{
  TabItemPtr    pCur;
  RectangleType rcCur = { { 0, mg_rcTabs.topLeft.y + 1 },
                          { 0, mg_rcTabs.extent.y + 2 } };
  Word  dx = 3;
  FntSetFont( 0 );
  for( pCur = mg_pTabHead; pCur; pCur = pCur->pNext )
    {
    CharPtr szTitle = pCur->pInfo->szTitle;
    Word    cbTitle;

    ErrFatalDisplayIf( !szTitle, "NULL Title" );

    cbTitle = StrLen( szTitle );

    RctCopyRectangle( &rcCur, &(pCur->rc) );
    pCur->rc.extent.x = FntCharsWidth( szTitle, cbTitle ) + dxText;
    pCur->rc.topLeft.x = dx;

    dx += pCur->rc.extent.x + 1;
    }
}

/*** TabManInit **************************************************************
*
*   Purpose:   Initializes the tab manager
*
*   Returns:   true if successful
*
*   Created:   3/30/1997  Eric Kenslow
*****************************************************************************/

Boolean TabManInit( RectanglePtr prc )
{
  Boolean fSuccess = true;
  TabInfoPtr  ptabT;
  Word    err, i;

  mg_rcTabsScreen = *prc;
  if( prc != &mg_rcTabs )
    mg_rcTabs = *prc;
  RctSetRectangle( &mg_rcDrag, 0, 0, 0, 0 );

  mg_hwinTabs = WinCreateWindow( prc, rectangleFrame, false, false, &err );
  if( err )
    {
    mg_hwinTabs = 0;
    fSuccess = false;
    }
  else
    {
    WinHandle hwinDraw = WinGetDrawWindow();
    WinSetDrawWindow( mg_hwinTabs );
    WinDisplayToWindowPt( &(mg_rcTabs.topLeft.x), &(mg_rcTabs.topLeft.y) );
    WinSetDrawWindow( hwinDraw );
    }

  mg_pTabHead = 0;
  i = 0;
  while( fSuccess && i < DBRecordCount( tabs ) &&
          (ptabT = DBGetRecord( tabs, i )) )
    {
    fSuccess = AddTab( ptabT );
    i++;
    }

  if( fSuccess && !mg_pTabHead )
    {
    UInt  i;
    char  szDefTab[ 256 ];
    for( i = 0; fSuccess && i < 3; i++ )
      {
      SysCopyStringResource( szDefTab, idStrDefTabs + i );
      fSuccess = TabManNewTab( szDefTab );
      }
    }

  InternalScroll( g_prefs.dxTabScroll, false );

  return fSuccess;
}

/*** TabManCleanup ***********************************************************
*
*   Purpose:   Cleans up after the tab manager
*
*   Returns:   Nothing
*
*   Created:   3/30/1997  Eric Kenslow
*****************************************************************************/

void TabManCleanup( void )
{
  Byte        i = 0;
  TabItemPtr  pitem = mg_pTabHead;
  mg_pTabHead = 0;
  while( pitem )
    {
    TabItemPtr  pT = pitem;
    pitem = pitem->pNext;
    DBReleaseRecord( tabs, i++, pT->pInfo );
    MemPtrFree( pT );
    }
  WinDeleteWindow( mg_hwinTabs, false );
}

/*** TabManDrawTabs **********************************************************
*
*   Purpose:   Draws all the tabs
*
*   Returns:   Nothing
*
*   Created:   3/30/1997  Eric Kenslow
*****************************************************************************/

void TabManDrawTabs(  )
{
  Byte        i;
  TabItemPtr  pCur;
  WinHandle hwinDraw = WinGetDrawWindow();

  WinSetDrawWindow( mg_hwinTabs );

  WinEraseRectangle( &mg_rcTabs, 0 );

  WinDrawLine( mg_rcTabs.topLeft.x,
               mg_rcTabs.topLeft.y + mg_rcTabs.extent.y - 1,
               mg_rcTabs.topLeft.x + mg_rcTabs.extent.x,
               mg_rcTabs.topLeft.y + mg_rcTabs.extent.y - 1 );

  FntSetFont( 0 );
  for( i = 0, pCur = mg_pTabHead; pCur; i++, pCur = pCur->pNext )
    {
    RectangleType rc;
    CharPtr szTitle = pCur->pInfo->szTitle;

    ErrFatalDisplayIf( !pCur->pInfo, "NULL Tab Info" );
    ErrFatalDisplayIf( !szTitle, "NULL String" );

    WinSetUnderlineMode( noUnderline );

    RctCopyRectangle( &(pCur->rc), &rc );
    if( g_prefs.iTabCur == i )
      {
      rc.topLeft.x -= 2;
      rc.extent.x  += 4;
      WinEraseRectangle( &rc, 3 );
      WinSetUnderlineMode( solidUnderline );
      }

    WinDrawRectangleFrame( roundFrame, &rc );
    WinDrawChars( szTitle, StrLen( szTitle ),
                   pCur->rc.topLeft.x + dxText,
                   pCur->rc.topLeft.y + dyText );

    if( g_prefs.iTabCur == i &&
        rc.topLeft.x + rc.extent.x > 0 )
      {
      RectangleType rcClip;
      RctCopyRectangle( &(mg_rcTabs), &rcClip );
      rcClip.topLeft.x  = rc.topLeft.x + rc.extent.x + 1;
      WinSetClip( &rcClip );
      }
    }

  WinResetClip();
  WinSetUnderlineMode( noUnderline );
  WinSetDrawWindow( hwinDraw );
}

/*** TabManHitTest ***********************************************************
*
*   Purpose:   Hit-tests the tabs
*
*   Returns:   Index of the tab at the given position, or (Byte)-1 if none
*
*   Created:   4/9/1997  Eric Kenslow
*****************************************************************************/

Byte TabManHitTest( SWord x, SWord y )
{
  Byte  iTab = (Byte)-1, i;
  TabItemPtr  pCur;
  if( !RctPtInRectangle( x, y, &mg_rcTabs ) )
    return iTab;

  FntSetFont( 0 );
  for( i = 0, pCur = mg_pTabHead; pCur; i++, pCur = pCur->pNext )
    {
    RectangleType rc;
    RctCopyRectangle( &(pCur->rc), &rc );
    RctOffsetRectangle( &rc, mg_rcTabsScreen.topLeft.x,
                             mg_rcTabsScreen.topLeft.y );
    if( RctPtInRectangle( x, y, &rc ) )
      {
      iTab = i;
      break;
      }
    }
  return iTab;
}

/*** TabManDelete ************************************************************
*
*   Purpose:   Deletes the current tab
*
*   Returns:   true if successful
*
*   Created:   4/12/1997  Eric Kenslow
*****************************************************************************/

Boolean TabManDelete( void )
{
  Byte        iTab  = g_prefs.iTabCur;
  TabItemPtr  pKill = mg_pTabHead, pPrev = 0;

  if( !(mg_pTabHead->pNext) )
    return false; // Need at least 1 tab

  while( iTab-- )
    {
    pPrev = pKill;
    pKill = pKill->pNext;
    }

  // Remove the item from our tab list
  if( pPrev )
    pPrev->pNext = pKill->pNext;
  else
    mg_pTabHead = mg_pTabHead->pNext;

  // Now remove it from our tab database
  DBReleaseRecord( tabs, g_prefs.iTabCur, pKill->pInfo );
  DBDeleteRecord( tabs, g_prefs.iTabCur );
  MemPtrFree( pKill );

  CalcTabRects();
  g_prefs.dxTabScroll = 0;

  return true;
}

/*** TabManGetName ***********************************************************
*
*   Purpose:   Gets the name of the current tab
*
*   Returns:   Pointer to the name
*
*   Created:   4/12/1997  Eric Kenslow
*****************************************************************************/

CharPtr TabManGetName( void )
{
  TabItemPtr  pCur = mg_pTabHead;
  Byte        iTab = g_prefs.iTabCur;
  while( iTab-- )
    pCur = pCur->pNext;
  return pCur->pInfo->szTitle;
}

/*** TabManSetName ***********************************************************
*
*   Purpose:   Sets the name of the current tab
*
*   Returns:   Nothing
*
*   Created:   4/12/1997  Eric Kenslow
*****************************************************************************/

void TabManSetName( CharPtr szName )
{
  TabItemPtr  pCur = mg_pTabHead;
  Byte        iTab = g_prefs.iTabCur;

  if( StrLen( szName ) == 0 )
    return ;

  while( iTab-- )
    pCur = pCur->pNext;
  // REVIEW: Pointer arithmetic
  DmStrCopy( pCur->pInfo, (ULong)pCur->pInfo->szTitle - (ULong)pCur->pInfo,
              szName );
  CalcTabRects();
  g_prefs.dxTabScroll = 0;
}

/*** TabManNewTab ************************************************************
*
*   Purpose:   Creates a new tab
*
*   Returns:   true if successful
*
*   Created:   4/13/1997  Eric Kenslow
*****************************************************************************/

Boolean TabManNewTab( CharPtr szTitle )
{
  Boolean fSuccess = false;
  TabInfoType infoT;
  TabInfoPtr  ptabT;
  UInt        iRec;

  StrCopy( infoT.szTitle, szTitle );
  iRec = dbAddAtEnd;
  DBAddRecord( tabs, &iRec, &infoT, sizeof( TabInfoType ) );
  ptabT = DBGetRecord( tabs, iRec );

  if( ptabT )
    fSuccess = AddTab( ptabT );
  CalcTabRects();
  g_prefs.dxTabScroll = 0;

  ErrFatalDisplayIf( !fSuccess, "Failed to create tab" );

  return fSuccess;
}

/*** TabManNextTab ***********************************************************
*
*   Purpose:   Switches to the next tab in the list
*
*   Returns:   Nothing
*
*   Created:   4/18/1997  Eric Kenslow
*****************************************************************************/

void TabManNextTab( void )
{
  UInt        iTabCur = ++g_prefs.iTabCur;
  TabItemPtr  pCur = mg_pTabHead;
  while( iTabCur-- )
    pCur = pCur->pNext;
  if( !pCur )
    g_prefs.iTabCur = 0;
}

/*** TabManDrag **************************************************************
*
*   Purpose:   Gives visual feedback of dragging over a tab
*
*   Returns:   Nothing
*
*   Created:   4/21/1997  Eric Kenslow
*****************************************************************************/

void TabManDrag( SWord x, SWord y )
{
  Byte  i = TabManHitTest( x, y );
  // Clear the rectangle
  WinInvertRectangle( &mg_rcDrag, 3 );
  if( i != (Byte)-1 )
    {
    TabItemPtr  pCur = mg_pTabHead;
    while( i-- )
      pCur = pCur->pNext;

    RctCopyRectangle( &(pCur->rc), &mg_rcDrag );
    RctOffsetRectangle( &mg_rcDrag, mg_rcTabsScreen.topLeft.x,
                                    mg_rcTabsScreen.topLeft.y );
    WinInvertRectangle( &mg_rcDrag, 3 );
    }
  else
    RctSetRectangle( &mg_rcDrag, 0, 0, 0, 0 );
}

/*** TabManMoveTab ***********************************************************
*
*   Purpose:   Changes a tab's index
*
*   Returns:   Nothing
*
*   Created:   4/21/1997  Eric Kenslow
*****************************************************************************/

void TabManMoveTab( Byte iOldPos, Byte iNewPos )
{
  ErrFatalDisplayIf( iOldPos == (Byte)-1, "oh no!" );
  ErrFatalDisplayIf( iNewPos == (Byte)-1, "OH NO!" );

  DBMoveRecord( tabs, iOldPos, iNewPos );

  TabManCleanup();
  TabManInit( &mg_rcTabsScreen );
}

/*** TabManScroll ************************************************************
*
*   Purpose:   Scrolls the tabs
*
*   Returns:   Nothing
*
*   Created:   4/26/1997  Eric Kenslow
*****************************************************************************/

Boolean TabManScroll( DirectionType dir )
{
  if( dir == left )
    InternalScroll( mg_rcTabs.extent.x / 2, true );
  else
    InternalScroll( -(mg_rcTabs.extent.x / 2), true );
  return true;
}

/*** TabManCanScroll *********************************************************
*
*   Purpose:   Queries if we can scroll the tabs
*
*   Returns:   Nothing
*
*   Created:   4/26/1997  Eric Kenslow
*****************************************************************************/

Boolean TabManCanScroll( DirectionType dir )
{
  if( dir == left )
    {
    return g_prefs.dxTabScroll < 0;
    }
  else
    {
    TabItemPtr  pCur = mg_pTabHead;
    while( pCur )
      {
      if( pCur->rc.topLeft.x + pCur->rc.extent.x > mg_rcTabs.extent.x )
        return true;
      pCur = pCur->pNext;
      }
    return false;
    }
}

